# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-03-09 17:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inventory', '0002_auto_20170623_1716'),
    ]

    operations = [
        migrations.AddField(
            model_name='manufacturer',
            name='rep_email',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='vendor',
            name='rep_email',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-23 21:16
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('inventory', '0001_squashed_0003_auto_20150626_1807'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inventory.Category'),
        ),
        migrations.AlterField(
            model_name='item',
            name='manufacturer',
            field=models.ForeignKey(blank=True, help_text='leave blank if unknown or same as vendor', null=True, on_delete=django.db.models.deletion.SET_NULL, to='inventory.Manufacturer'),
        ),
        migrations.AlterField(
            model_name='item',
            name='parent_item',
            field=models.ForeignKey(blank=True, help_text='example: for printer cartriges, select printer', null=True, on_delete=django.db.models.deletion.SET_NULL, to='inventory.Item'),
        ),
        migrations.AlterField(
            model_name='item',
            name='unit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inventory.Unit'),
        ),
        migrations.AlterField(
            model_name='item',
            name='vendor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inventory.Vendor'),
        ),
        migrations.AlterField(
            model_name='order',
            name='ordered_by',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='order',
            name='ptao',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='inventory.PTAO'),
        ),
    ]



from django.contrib import admin

from inventory.models import (
    Account,
    Category,
    Item,
    Manufacturer,
    Order,
    OrderItem,
    Unit,
    Vendor,
)


class OrderItemInline(admin.StackedInline):
    model = OrderItem
    extra = 1


class ItemAdmin(admin.ModelAdmin):
    date_hierarchy = "date_added"

    fieldsets = [
        (None, {"fields": ["name", "chem_formula", "category"]}),
        (
            "Vendor Information",
            {
                "fields": [
                    "vendor",
                    "catalog",
                    "manufacturer",
                    "manufacturer_number",
                    "size",
                    "unit",
                ]
            },
        ),
        (None, {"fields": ["parent_item", "comments"]}),
    ]

    list_display = (
        "name",
        "category",
        "vendor",
        "catalog",
        "date_added",
    )
    list_filter = ("category", "vendor", "manufacturer", "date_added")
    search_fields = ("name", "chem_formula", "manufacturer_number", "comments")
    inlines = (OrderItemInline,)


admin.site.register(Item, ItemAdmin)


class OrderAdmin(admin.ModelAdmin):
    date_hierarchy = "order_date"
    fields = (
        "name",
        "order_date",
        "ordered_by",
        "ordered",
        "account",
    )
    list_display = ("name", "item_count", "order_date", "ordered", "account")
    list_filter = ("ordered", "account", "ordered_by")
    search_fields = ("name",)
    inlines = (OrderItemInline,)


admin.site.register(Order, OrderAdmin)


class OrderItemAdmin(admin.ModelAdmin):
    date_hierarchy = "date_arrived"
    fields = (
        "units_purchased",
        "cost",
        "date_arrived",
        "serial",
        "uva_equip",
        "location",
        "expiry_years",
        "reconciled",
    )
    list_display = ("name", "order_date", "date_arrived", "total_price", "location")
    list_filter = ("item__name", "order__order_date", "date_arrived", "location")


admin.site.register(OrderItem, OrderItemAdmin)


class AccountAdmin(admin.ModelAdmin):
    fields = ("code", "description", "expires")
    list_display = fields


admin.site.register(Account, AccountAdmin)

for model in (Category, Unit, Manufacturer, Vendor):
    admin.site.register(model)



from django.urls import re_path

from inventory import views

app_name = "inventory"
urlpatterns = [
    re_path(r"^$", views.index, name="index"),
    re_path(r"^orders/$", views.OrderList.as_view(), name="orders"),
    re_path(r"^orders/new/$", views.OrderEntry.as_view(), name="new_order"),
    re_path(r"^orders/(?P<pk>\d+)/$", views.OrderView.as_view(), name="order"),
    re_path(r"^items/$", views.ItemList.as_view(), name="items"),
    re_path(r"^items/new/$", views.ItemEntry.as_view(), name="new_item"),
    re_path(r"^items/(?P<pk>\d+)/$", views.ItemView.as_view(), name="item"),
    re_path(
        r"^items/(?P<pk>\d+)/order/$",
        views.OrderItemEntry.as_view(),
        name="add_item_to_order",
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('inventory', '0002_auto_20150626_1805'),
    ]

    operations = [
        migrations.AlterField(
            model_name='manufacturer',
            name='rep',
            field=models.CharField(null=True, max_length=128, blank=True),
        ),
        migrations.AlterField(
            model_name='vendor',
            name='lookup_url',
            field=models.CharField(null=True, help_text='url pattern to look up catalog number', max_length=128, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
# -*- mode: python -*-
import datetime

from django.contrib.auth.models import User
from django.db import models
from django.urls import reverse


class Category(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=45, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "categories"
        ordering = ["name"]


class Unit(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=45, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class Manufacturer(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=64, unique=True)
    url = models.CharField(max_length=64, blank=True, null=True)
    lookup_url = models.CharField(
        max_length=64,
        blank=True,
        null=True,
        help_text="url pattern to look up part number",
    )
    rep = models.CharField(max_length=128, blank=True, null=True)
    rep_phone = models.CharField(max_length=16, blank=True, null=True)
    rep_email = models.CharField(max_length=64, blank=True, null=True)
    support_phone = models.CharField(max_length=16, blank=True, null=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class Vendor(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=64, unique=True)
    url = models.CharField(max_length=64, blank=True, null=True)
    lookup_url = models.CharField(
        max_length=128,
        blank=True,
        null=True,
        help_text="url pattern to look up catalog number",
    )
    phone = models.CharField(max_length=16, blank=True, null=True)
    rep = models.CharField(max_length=45, blank=True, null=True)
    rep_phone = models.CharField(max_length=16, blank=True, null=True)
    rep_email = models.CharField(max_length=64, blank=True, null=True)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class Account(models.Model):
    id = models.AutoField(primary_key=True)
    code = models.CharField(max_length=64, unique=True)
    description = models.CharField(max_length=128)
    expires = models.DateField(blank=True, null=True)

    def __str__(self):
        return "%s (%s)" % (self.description, self.code)

    class Meta:
        ordering = ["code"]
        verbose_name = "Account"
        verbose_name_plural = "Accounts"


class Item(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=128)
    chem_formula = models.CharField(
        "Chemical formula", max_length=45, blank=True, null=True
    )

    vendor = models.ForeignKey(Vendor, on_delete=models.PROTECT)
    catalog = models.CharField("Catalog number", max_length=45, blank=True, null=True)
    manufacturer = models.ForeignKey(
        "Manufacturer",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        help_text="leave blank if unknown or same as vendor",
    )
    manufacturer_number = models.CharField(max_length=45, blank=True, null=True)
    size = models.DecimalField(
        "Size of unit", max_digits=10, decimal_places=2, blank=True, null=True
    )
    unit = models.ForeignKey(Unit, on_delete=models.PROTECT)
    category = models.ForeignKey(Category, on_delete=models.PROTECT)
    date_added = models.DateField(auto_now_add=True)
    parent_item = models.ForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        help_text="example: for printer cartriges, select printer",
    )
    comments = models.TextField(blank=True)

    def __str__(self):
        return self.name

    def unit_size(self):
        if self.unit.name == "each":
            return self.unit.name
        return "%s%s%s" % (
            self.size or "",
            "" if str(self.unit).startswith("/") else " ",
            self.unit,
        )

    def total_price(self):
        return (self.cost or 0) * self.units_purchased

    def vendor_url(self):
        try:
            return self.vendor.lookup_url % self.catalog
        except (AttributeError, TypeError):
            return None

    def mfg_url(self):
        try:
            return self.manufacturer.lookup_url % self.manufacturer_number
        except (AttributeError, TypeError):
            return None

    def get_absolute_url(self):
        return reverse("inventory:item", kwargs={"pk": self.pk})

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=["vendor", "catalog"], name="unique_vendor_catalog_number"
            )
        ]


class Order(models.Model):
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=64)
    created = models.DateTimeField(auto_now_add=True)
    items = models.ManyToManyField(Item, through="OrderItem")
    account = models.ForeignKey(
        Account, blank=True, null=True, on_delete=models.SET_NULL
    )
    ordered = models.BooleanField(default=False)
    order_date = models.DateField(default=datetime.date.today)
    ordered_by = models.ForeignKey(User, on_delete=models.PROTECT)

    def __str__(self):
        if self.ordered:
            status = self.order_date
        else:
            status = "in progress"
        return "%s (%s)" % (self.name, status)

    @property
    def item_count(self):
        return self.items.count

    def get_absolute_url(self):
        return reverse("inventory:order", kwargs={"pk": self.pk})

    class Meta:
        ordering = ["-created"]


class OrderItem(models.Model):
    id = models.AutoField(primary_key=True)
    item = models.ForeignKey(Item, on_delete=models.CASCADE)
    order = models.ForeignKey(Order, on_delete=models.CASCADE)

    units_purchased = models.IntegerField()
    cost = models.DecimalField(
        "Cost per unit", max_digits=10, decimal_places=2, blank=True, null=True
    )

    date_arrived = models.DateField(blank=True, null=True)
    serial = models.CharField("Serial number", max_length=45, blank=True, null=True)
    uva_equip = models.CharField(
        "UVa equipment number", max_length=32, blank=True, null=True
    )
    location = models.CharField(
        max_length=45,
        blank=True,
        null=True,
        help_text="example: -80 freezer, refrigerator, Gilmer 283",
    )
    expiry_years = models.DecimalField(
        "Warranty or Item expiration (y)",
        max_digits=4,
        decimal_places=2,
        blank=True,
        null=True,
    )

    reconciled = models.BooleanField(default=False)

    def total_price(self):
        return (self.cost or 0) * self.units_purchased

    def name(self):
        return self.item.name

    def order_date(self):
        return self.order.order_date

    def __str__(self):
        return "%s [%s]" % (self.item.name, self.order.order_date)

    class Meta:
        db_table = "inventory_order_items"



# -*- coding: utf-8 -*-
# -*- mode: python -*-
import datetime
from django import forms
from django.contrib.auth.models import User

from inventory.models import Account, Order, Vendor, Item, OrderItem


class NewOrderForm(forms.ModelForm):
    name = forms.CharField(label="Order Name")
    ordered_by = forms.ModelChoiceField(queryset=User.objects.filter(is_active=True))
    account = forms.ModelChoiceField(
        queryset=Account.objects.exclude(expires__lt=datetime.date.today()),
        required=False,
    )

    class Meta:
        model = Order
        fields = ["name", "ordered_by", "account"]


class NewItemForm(forms.ModelForm):
    name = forms.CharField(label="Item Name")
    vendor = forms.ModelChoiceField(queryset=Vendor.objects.all(), required=True)
    catalog = forms.CharField(label="Vendor catalog number")
    manufacturer_number = forms.CharField(
        label="Manufacturer part number", required=False
    )

    class Meta:
        model = Item
        fields = [
            "name",
            "category",
            "vendor",
            "catalog",
            "manufacturer",
            "manufacturer_number",
            "size",
            "unit",
            "parent_item",
            "comments",
        ]


class NewOrderItemForm(forms.ModelForm):
    order = forms.ModelChoiceField(
        queryset=Order.objects.filter(ordered=False),
        required=False,
        label="Choose an in-progress order",
    )
    units_purchased = forms.IntegerField(label="Number of units to order")
    cost = forms.DecimalField(label="Current price per unit")

    class Meta:
        model = OrderItem
        fields = ["order", "units_purchased", "cost"]



# -*- coding: utf-8 -*-
# -*- mode: python -*-
from django.shortcuts import get_object_or_404, redirect
import django_filters as filters
from django.db.models import Q
from django.shortcuts import render
from django.views import generic
from django_filters.views import FilterView

from inventory.models import Item, Order
from inventory.forms import NewOrderForm, NewItemForm, NewOrderItemForm


def index(request):
    return render(request, "inventory/index.html")


class PaginatedFilterView(FilterView):
    paginate_by = 25

    def get_context_data(self, **kwargs):
        # This strips the page from the query parameters; otherwise pagination leads to ever-growing URL
        context = super().get_context_data(**kwargs)
        context["query"] = self.request.GET.copy()
        try:
            del context["query"]["page"]
        except KeyError:
            pass
        return context


class OrderFilter(filters.FilterSet):
    name = filters.CharFilter(field_name="name", lookup_expr="icontains", label="Name")
    ordered_by = filters.CharFilter(
        field_name="ordered_by__username", lookup_expr="istartswith", label="Ordered by"
    )
    account = filters.CharFilter(
        field_name="account__description", lookup_expr="icontains", label="Account"
    )
    # example of a drop-down filter. I don't like how these look though
    # ordered_by = filters.ModelChoiceFilter(
    #     field_name="ordered_by",
    #     queryset=User.objects.all(),
    #     label="User",
    #     to_field_name="username",
    # )

    class Meta:
        model = Order
        fields = ["name", "ordered_by", "account"]


class ItemFilter(filters.FilterSet):
    name = filters.CharFilter(
        field_name="name", lookup_expr="icontains", label="Description"
    )
    vendor_or_mfg = filters.CharFilter(
        method="by_vendor_or_mfg", label="Vendor/Manufacturer"
    )
    catalog_or_part = filters.CharFilter(
        method="by_catalog_or_part", label="Catalog/Part Number"
    )
    category = filters.CharFilter(
        field_name="category__name", lookup_expr="istartswith", label="Category"
    )

    def by_vendor_or_mfg(self, queryset, name, value):
        return queryset.filter(
            Q(vendor__name__icontains=value) | Q(manufacturer__name__icontains=value)
        )

    def by_catalog_or_part(self, queryset, name, value):
        return queryset.filter(
            Q(catalog__icontains=value) | Q(manufacturer_number__icontains=value)
        )

    class Meta:
        model = Order
        fields = [
            "name",
            "vendor_or_mfg",
            "catalog_or_part",
            "category",
        ]


class OrderList(PaginatedFilterView):
    model = Order
    filterset_class = OrderFilter
    template_name = "inventory/order_list.html"
    context_object_name = "order_list"


class OrderView(generic.DetailView):
    model = Order
    template_name = "inventory/order.html"

    def get_context_data(self, **kwargs):
        context = super(OrderView, self).get_context_data(**kwargs)
        context["orderitem_list"] = context["order"].orderitem_set.order_by(
            "item__vendor"
        )
        return context


class OrderEntry(generic.FormView):
    template_name = "inventory/order_entry.html"
    form_class = NewOrderForm

    def get_initial(self):
        initial = super().get_initial()
        initial["ordered_by"] = self.request.user
        return initial

    def form_valid(self, form):
        order = form.save(commit=False)
        order.ordered = False
        order.save()
        return redirect("inventory:order", pk=order.id)


class ItemList(PaginatedFilterView):
    model = Item
    filterset_class = ItemFilter
    template_name = "inventory/item_list.html"
    context_object_name = "item_list"

    def get_queryset(self):
        qs = Item.objects.filter(**self.kwargs)
        return qs.order_by("-date_added")


class ItemView(generic.DetailView, generic.FormView):
    model = Item
    template_name = "inventory/item.html"
    form_class = NewOrderItemForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["lineitems"] = context["item"].orderitem_set.order_by(
            "-order__order_date"
        )
        return context


class ItemEntry(generic.FormView):
    template_name = "inventory/item_entry.html"
    form_class = NewItemForm

    def form_valid(self, form):
        item = form.save()
        return redirect("inventory:item", pk=item.id)


class OrderItemEntry(generic.FormView):
    """Order an item (associate it with an order)"""

    template_name = "inventory/orderitem_entry.html"
    form_class = NewOrderItemForm

    def form_valid(self, form):
        item_id = self.kwargs["pk"]
        form.instance.item = get_object_or_404(Item, pk=item_id)
        form.instance.reconciled = False
        oitem = form.save()
        return redirect("inventory:order", pk=oitem.order.id)
